/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.search.elasticsearch.internal.facet;

import com.liferay.portal.kernel.search.facet.collector.DefaultTermCollector;
import com.liferay.portal.kernel.search.facet.collector.TermCollector;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author André de Oliveira
 */
public class TermCollectorHolder {

	public TermCollectorHolder(int size) {
		_termCollectors = new ArrayList<>(size);
		_termCollectorsByName = new HashMap<>(size);
	}

	public void add(String term, int frequency) {
		TermCollector termCollector = new DefaultTermCollector(term, frequency);

		_termCollectors.add(termCollector);

		_termCollectorsByName.put(term, termCollector);
	}

	public TermCollector getTermCollector(String term) {
		TermCollector termCollector = _termCollectorsByName.get(term);

		if (termCollector != null) {
			return termCollector;
		}

		return new DefaultTermCollector(term, 0);
	}

	public List<TermCollector> getTermCollectors() {
		return _termCollectors;
	}

	private final List<TermCollector> _termCollectors;
	private final Map<String, TermCollector> _termCollectorsByName;

}