/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.search.internal.buffer;

import com.liferay.portal.search.buffer.IndexerRequest;
import com.liferay.portal.search.buffer.IndexerRequestBuffer;
import com.liferay.portal.search.buffer.IndexerRequestBufferOverflowHandler;
import com.liferay.portal.search.configuration.IndexerRegistryConfiguration;

/**
 * @author Bryan Engler
 * @author Michael C. Han
 */
public class IndexerRequestBufferHandler {

	public IndexerRequestBufferHandler(
		IndexerRequestBufferOverflowHandler indexerRequestBufferOverflowHandler,
		IndexerRegistryConfiguration indexerRegistryConfiguration) {

		_indexerRequestBufferOverflowHandler =
			indexerRequestBufferOverflowHandler;

		_indexerRegistryConfiguration = indexerRegistryConfiguration;
	}

	public void bufferRequest(
			IndexerRequest indexerRequest,
			IndexerRequestBuffer indexerRequestBuffer)
		throws Exception {

		if (!BufferOverflowThreadLocal.isOverflowMode()) {
			indexerRequestBuffer.add(
				indexerRequest, _indexerRequestBufferOverflowHandler,
				_indexerRegistryConfiguration.maxBufferSize());
		}
		else {
			indexerRequestBuffer.remove(indexerRequest);

			indexerRequest.execute();
		}
	}

	private final IndexerRegistryConfiguration _indexerRegistryConfiguration;
	private final IndexerRequestBufferOverflowHandler
		_indexerRequestBufferOverflowHandler;

}